/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef ASYNCHRONOUS_SIGNALS_INVOKE_HPP
#define ASYNCHRONOUS_SIGNALS_INVOKE_HPP 1

#include <type_traits>
#include <tuple>

namespace asynchronous
{

  namespace signals
  {

    /// \brief Invoke a function with an unpacked tuple parameter pack
    ///
    /// \note See http://en.cppreference.com/w/cpp/utility/integer_sequence.
    ///
    class invoke
    {
      template < typename F, typename Tuple, std::size_t ... indexes >
      static decltype(auto) invoke_helper (F&& f, Tuple&& tuple, std::index_sequence<indexes...>)
      {
	return f(std::get<indexes>(std::forward<Tuple>(tuple))...);
      }

    public:
      template < typename F, typename Tuple >
      static decltype(auto) apply (F&& f, Tuple&& tuple)
      {
	constexpr auto Size = std::tuple_size<std::decay_t<Tuple>>::value;
	return invoke_helper(std::forward<F>(f),
			     std::forward<Tuple>(tuple),
			     std::make_index_sequence<Size>{});
      }
    };

  } // namespace signals

} // namespace asynchronous

#endif
