/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef ASYNCHRONOUS_SIGNALS_PRODUCT_COMBINER_HPP
#define ASYNCHRONOUS_SIGNALS_PRODUCT_COMBINER_HPP 1

#include <cassert>
#include <boost/log/sources/severity_logger.hpp>
#include <boost/log/trivial.hpp>
#include <boost/optional.hpp>
#include <logging/channel_log.hpp>
#include <logging/global_log.hpp>

namespace asynchronous
{

  namespace signals
  {

    template < typename T >
    class product_combiner
    {
      using self = product_combiner;
      using optional_value = boost::optional<T>;
      using log_type = logging::channel_log;

    public:
      auto result () const
      {
	using namespace boost::log::trivial;
	auto result = value_ ? *value_ : 1;
	GLOBAL_LOG(log_,trace) << "result()const: result=" << result;
	return result;
      }

      void merge (self const& other)
      {
	using namespace boost::log::trivial;
	GLOBAL_LOG(log_,trace) << "merge(self const&)";
	// std::cout << "XXX - asynchronous::signals::product_combiner::merge()" << std::endl;
	if (value_ && other.value_)
	  {
	    GLOBAL_LOG(log_,trace) << "merge(self const&): *value_=" << *value_ << ", *other.value_=" << *other.value_;
	    *value_ *= *other.value_;
	  }
	else if (other.value_)
	  {
	    GLOBAL_LOG(log_,trace) << "merge(self const&): *other.value_=" << *other.value_;
	    value_ = other.value_;
	  }
	GLOBAL_LOG(log_,trace) << "merge(self const&): *value_=" << *value_;
      }

      void operator () (T t)
      {
	using namespace boost::log::trivial;
	if (value_)
	  {
	    GLOBAL_LOG(log_,trace) << "operator()(T): *value_=" << *value_ << ", t=" << t;
	    *value_ *= t;
	  }
	else
	  {
	    GLOBAL_LOG(log_,trace) << "operator()(T): t=" << t;
	    value_ = t;
	  }
      }

    private:
      optional_value value_;
      mutable log_type log_ = "asynchronous::signals::product_combiner";
    };

  } // namespace signals

} // namespace asynchronous

#endif
