/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef ASYNCHRONOUS_SIGNALS_SLOT_ITERATOR_HPP
#define ASYNCHRONOUS_SIGNALS_SLOT_ITERATOR_HPP 1

#include <boost/iterator/filter_iterator.hpp>

namespace asynchronous
{

  namespace signals
  {

    namespace detail
    {

      struct slot_iterator_predicate
      {
	template < typename T >
	auto operator () (T const& t) const
	{ return static_cast<bool>(t); }
      };

    } // namespace detail

    template < typename Iterator >
    using slot_iterator_t = boost::filter_iterator<detail::slot_iterator_predicate,Iterator>;

    template < typename Iterator >
    auto make_slot_iterator (Iterator begin, Iterator end)
    {
      using boost::make_filter_iterator;
      return make_filter_iterator<detail::slot_iterator_predicate>(begin,end);
    }

  } // namespace signals

} // namespace asynchronous

#endif
