/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_TYPE_SIGNAL_HPP
#define GRAPH_MODEL_TYPE_SIGNAL_HPP 1

#include <graph_model/signals/signal_fwd.hpp>
#include <use_default.hpp>

namespace graph_model
{

  namespace type
  {

    /// \brief Deduce the signal type
    /// \tparam Signature The type of signature of the signal
    /// \tparam Combiner The type of combiner for the signal
    /// \tparam SignalTag The tag identifying the type of signal
    /// \tparam SlotContainer A tag or slot container type
    ///
    /// This class may be specialized to select specific signal types.
    /// However, the specializations must select signal types that are
    /// appropriate for use by the library.
    ///
    template < typename Signature
	       , typename Combiner = use_default<>
	       , typename SignalTag = use_default<>
	       , template < typename ... > class SlotContainer = use_default
	       >
    struct signal
    {
      using type = graph_model::signals::signal<Signature,Combiner,SignalTag,SlotContainer>;
    };

    template < typename Signature
	       , typename Combiner = use_default<>
	       , typename SignalTag = use_default<>
	       , template < typename ... > class SlotContainer = use_default
	       >
    using signal_t = typename signal<Signature,Combiner,SignalTag,SlotContainer>::type;

  } // namespace type

} // namespace graph_model

#endif
