/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_VERTEX_EXPRESSION_TERMINAL_HPP
#define GRAPH_MODEL_VERTEX_EXPRESSION_TERMINAL_HPP 1

#include <boost/yap/algorithm.hpp>
#include <graph_model/vertex/vertex_expression.hpp>

namespace graph_model
{

  namespace vertex
  {

    /// \brief Type alias for an expression terminal
    ///
    /// Overloads for \c transform_expression() take a const reference
    /// to an expression as an argument.  This type alias simplifies
    /// the specification of that argument for the common case of the
    /// expression being a terminal.
    ///
    template < typename Expr >
    using expression_terminal = vertex_expression<boost::yap::expr_kind::terminal,boost::hana::tuple<Expr>> const&;

    template < typename Terminal, typename ... Args >
    auto evaluate_terminal (Terminal&& terminal, Args&& ... args)
    {
      using boost::yap::get_c;
      return get_c<0>(terminal)(std::forward<Args>(args)...);
    }

  } // namespace vertex

} // namespace graph_model

#endif
