/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_VERTEX_SCALAR_DISTRIBUTION_OBSERVER_BASE_HPP
#define GRAPH_MODEL_VERTEX_SCALAR_DISTRIBUTION_OBSERVER_BASE_HPP 1

#include <graph_model/vertex/distribution_observer_base.hpp>
#include <type_traits/is_rvalue.hpp>

namespace graph_model
{

  namespace vertex
  {

    /// \brief Manage a scalar distribution observer
    ///
    /// Some vertex classes delegate their member functions to an
    /// observed scalar distribution.  This class manages the observer
    /// and allows reseting the observed distribution at runtime.  It
    /// is the caller's responsibility to set a distribution prior to
    /// use.
    ///
    template < typename Derived, typename ScalarDistribution >
    class scalar_distribution_observer_base
      : public distribution_observer_base<Derived,ScalarDistribution>
    {
      using base = distribution_observer_base<Derived,ScalarDistribution>;

    public:			// XXX - for CRTP only
      auto& scalar_distribution () const { return base::distribution(); }

    public:
      auto& reset_scalar_distribution () { return base::reset_distribution(); }

      template < typename E >
      auto reset_scalar_distribution (E&& distribution)
      { return base::reset_distribution(std::forward<E>(distribution)); }
    };

  } // namespace vertex

} // namespace graph_model

#endif
