/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_VERTEX_STRATEGY_BASIC_EXPRESSION_HPP
#define GRAPH_MODEL_VERTEX_STRATEGY_BASIC_EXPRESSION_HPP 1

#include <memory>
#include <graph_model/concept/expression_policy_concept.hpp>
#include <graph_model/vertex/strategy/expression_base.hpp>
#include <graph_model/vertex/strategy/policy_owner_base.hpp>
#include <graph_model/vertex/trapping_eval_base.hpp>

namespace graph_model
{

  namespace vertex
  {

    namespace strategy
    {

      template < typename Policy
		 , typename Result
		 >
      class basic_expression
	: public expression_base<Result>
	, public policy_owner_base<basic_expression<Policy,Result>,Policy>
      {
	using self = basic_expression;
	using base = expression_base<Result>;
	using policy_owner_base = policy_owner_base<basic_expression<Policy,Result>,Policy>;

	BOOST_CONCEPT_ASSERT((concept::expression_policy_concept<Policy,Result>));

      public:
	basic_expression () = default;
	explicit basic_expression (Policy const& policy) : policy_owner_base(policy) {}
	explicit basic_expression (Policy&& policy) : policy_owner_base(std::move(policy)) {}

      private:
	typename base::result_type
	eval_ () const override { return this->policy()(); }
      };

      template < typename Result >
      struct basic_expression< void,Result >
	: trapping_eval_base< basic_expression<void,Result>,
	  expression_base<Result
	  > >
      {};

      template < typename Result >
      auto make_basic_expression_policy_pointer ()
      {
	using basic_expression_type = basic_expression<void,Result>;
	return std::make_unique<basic_expression_type>();
      }

      template < typename Result, typename Policy >
      auto make_basic_expression_policy_pointer (Policy&& policy)
      {
	using policy_type = std::decay_t<Policy>;
	using basic_expression_type = basic_expression<policy_type,Result>;
	return std::make_unique<basic_expression_type>(std::forward<Policy>(policy));
      }

      template < typename Result >
      struct basic_expression_factory
      {
	using pointer = std::unique_ptr<expression_base<Result>>;

	static pointer apply ()
	{ return make_basic_expression_policy_pointer<Result>(); }

	template < typename Policy >
	static pointer apply (Policy&& policy)
	{ return make_basic_expression_policy_pointer<Result>(std::forward<Policy>(policy)); }
      };

    } // namespace strategy

  } // namespace vertex

} // namespace graph_model

#endif
