/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_VERTEX_SUMMARIZED_SCALAR_RANDOM_VARIABLE_FWD_HPP
#define GRAPH_MODEL_VERTEX_SUMMARIZED_SCALAR_RANDOM_VARIABLE_FWD_HPP 1

#include <graph_model/type_traits/is_vertex.hpp>
#include <traits/probability.hpp>
#include <traits/variate.hpp>
#include <use_default.hpp>

namespace graph_model
{

  namespace vertex
  {

    template < typename Variate
	       , typename Probability
	       , typename Stats = use_default<>
	       >
    class summarized_scalar_random_variable;

  } // namespace vertex

  namespace type_traits
  {

    template < typename ... T >
    struct is_vertex< ::graph_model::vertex::summarized_scalar_random_variable<T...> >
      : std::true_type
    {};

  } // namespace type_traits

} // namespace graph_model

namespace traits
{

  namespace defaults
  {

    template < typename Variate, typename Probability, typename Stats >
    struct probability< graph_model::vertex::summarized_scalar_random_variable<Variate,Probability,Stats> >
    {
      using type = probability_t<Probability>;
    };

    template < typename Variate, typename Probability, typename Stats >
    struct variate< graph_model::vertex::summarized_scalar_random_variable<Variate,Probability,Stats> >
    {
      using type = variate_t<Variate>;
    };

  } // namespace defaults

} // namespace traits

#endif
