/*
 * Copyright (c) 2017 Gregory Penn.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#define BOOST_TEST_MODULE gdal

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

template < typename ... T > class XXX;

#include <iostream>
#include <ogr_core.h>
#include <gdal/geographic_srs.hpp>

using std::cout;
using std::endl;
using namespace gdal;

BOOST_AUTO_TEST_CASE(geographic_srs_default_constructor)
{
  geographic_srs srs;
}

BOOST_AUTO_TEST_CASE(geographic_make_wgs84)
{
  auto wgs84 = make_wgs84();
}

BOOST_AUTO_TEST_CASE(geographic_srs_make_from_epsg)
{
  make_geographic_srs(2022);
}

BOOST_AUTO_TEST_CASE(geographic_srs_copy_constructor)
{
  auto wgs84 = make_wgs84();
  geographic_srs srs_copy(wgs84);
}

BOOST_AUTO_TEST_CASE(geographic_srs_move_constructor)
{
  auto wgs84 = make_wgs84();
  auto srs_moved = std::move(wgs84);
}

BOOST_AUTO_TEST_CASE(geographic_equality_comparable)
{
  auto srs1 = make_wgs84();
  auto srs1_copy = geographic_srs(srs1);

  auto srs2 = make_geographic_srs(2022);
  BOOST_CHECK(srs1 == srs1_copy);
  BOOST_CHECK(srs2 == srs2);
}

BOOST_AUTO_TEST_CASE(geographic_inequality_comparable)
{
  auto srs1 = make_wgs84();
  auto srs2 = make_geographic_srs(2022);
  BOOST_CHECK(srs1 != srs2);
}

BOOST_AUTO_TEST_CASE(geographic_srs_bool)
{
  auto wgs84 = make_wgs84();
  geographic_srs null_srs;

  BOOST_CHECK(static_cast<bool>(wgs84));
  BOOST_CHECK(!static_cast<bool>(null_srs));
}

BOOST_AUTO_TEST_CASE(geographic_srs_new)
{
  auto srs  = make_geographic_srs(3857);
  {geographic_srs srs_copy(srs);}
  geographic_srs srs_copy2(srs);
  auto srs_moved(std::move(srs));

}

BOOST_AUTO_TEST_CASE(dummy)
{
  auto srs  = make_geographic_srs(3857);
  geographic_srs srs_copy(srs);
}
