#
# Copyright (c) 2017 Brook Milligan.
#
# This file is part of the GraphModel Library.
#
#     The GraphModel Library is free software: you can redistribute it
#     and/or modify it under the terms of the GNU Affero General
#     Public License as published by the Free Software Foundation,
#     either version 3 of the License, or (at your option) any later
#     version.
#
#     The GraphModel Library is distributed in the hope that it will
#     be useful, but WITHOUT ANY WARRANTY; without even the implied
#     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#     See the GNU Affero General Public License for more details.
#
#     You should have received a copy of the GNU Affero General Public
#     License along with the GraphModel Library.  If not, see
#     <http://www.gnu.org/licenses/>.
#

library(ggplot2)

filename <- 'samples.dat'
pdfname <- 'samples.pdf'

sample.size <- 500
mean <- 1.5
sd <- 1
error <- 0.5

z <- rnorm(n=sample.size,mean=mean,sd=sd)
x <- rnorm(n=sample.size,mean=z,sd=error)
x <- x[-2 <= x & x <= 2]
data <- data.frame(samples=x)
write.table(data,filename,quote=FALSE,sep="\t",row.names=FALSE)

pdf(pdfname,width=7,height=3)
ggplot(data,aes(x=samples)) +
  theme_bw() +
  geom_histogram(color='gray',fill='gray') +
  geom_vline(aes(xintercept=mean(data$samples))) +
  geom_vline(aes(xintercept=mean),linetype='dashed')
dev.off()

str(data)
summary(data)
print(paste('samples:',sample.size))
print(paste('mean:',mean(data$samples)))
print(paste('var:',var(data$samples)))
print(paste('sd:',sd(data$samples)))

